@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import
 
-__version__ = '0.6.1'
+__version__ = '0.7'
 
-from .formatters import format_as_html, format_html_styles, format_as_text
+from .formatters import (
+    format_as_html,
+    format_html_styles,
+    format_as_text,
+    format_as_dict,
+)
 from .explain import explain_weights, explain_prediction
 from .sklearn import explain_weights_sklearn, explain_prediction_sklearn
 from .transform import transform_feature_names
@@ -15,6 +20,16 @@ except ImportError:
     pass  # IPython is not installed
 
 
+try:
+    from .formatters.as_dataframe import (
+        explain_weights_df, explain_weights_dfs,
+        explain_prediction_df, explain_prediction_dfs,
+        format_as_dataframe, format_as_dataframes,
+    )
+except ImportError:
+    pass  # pandas not available
+
+
 try:
     from .lightning import (
         explain_prediction_lightning,
