@@ -132,6 +132,12 @@ def explain_prediction(estimator, doc, **kwargs):
         an estimator or names defined in ``target_names`` parameter.
         Must not be given with ``top_targets`` argument.
 
+        In case of binary classification you can use this argument to
+        set the class which probability or score should be displayed, with
+        an appropriate explanation. By default a result for predicted class
+        is shown. For example, you can use ``targets=[True]`` to always show
+        result for a positive class, even if the predicted label is False.
+
         This argument may be supported or not, depending on estimator type.
 
     feature_names : list, optional
